@@ -26,7 +26,7 @@ import torch.nn as nn
 
 
 class AbstractRunner(ABC):
-    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, save_model=False, supervision_npy_file_suffix='_supervision.npy', macro_supervision_npy_file_suffix='_macro_supervision.npy', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', shortcut_features_names_saving_suffix = '_shortcut_features_names.npy', shortcut_features_saving_suffix='_shortcut_features.npy',returns_pkl_file_name='returns.pkl',  local_root_directory='../data/', user = 'napoleon', n_start = 252, maximum_precision = False, shortcut_features = False):
+    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, save_model=False, supervision_npy_file_suffix='_supervision.npy', macro_supervision_npy_file_suffix='_macro_supervision.npy', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', shortcut_features_names_saving_suffix = '_shortcut_features_names.npy', shortcut_features_saving_suffix='_shortcut_features.npy',returns_pkl_file_name='returns.pkl',  local_root_directory='../data/', user = 'napoleon', n_start = 252, maximum_precision = False, shortcut_features = False, transaction_costs = None):
         super().__init__()
         self.local_root_directory=local_root_directory
         self.supervision_npy_file_suffix=supervision_npy_file_suffix
@@ -45,6 +45,7 @@ class AbstractRunner(ABC):
         self.running_date = running_date
         self.starting_date = starting_date
         self.maximum_precision = maximum_precision
+        self.transaction_costs = transaction_costs
 
     @abstractmethod
     def runTrial(self,saver, seed, sup, layers, epochs, n_past_features, n, s, whole_history, advance_feature, advance_signal,
@@ -62,11 +63,9 @@ class SimpleRunner(AbstractRunner):
             return
 
         saving_key = saver.create_saving_key(seed, sup, layers, epochs, n_past_features, n, s, feature_type, activation_type, convolution, lr_start, lr_type, low_bound, up_bound, normalize)
-
-
-
         print('Launching computation with parameters : '+saving_key)
 
+
         ## checking if the computation has already been done
         run_found, found_table_number = saver.checkRunExistence(saving_key)
         if run_found:
@@ -292,11 +291,29 @@ class SimpleRunner(AbstractRunner):
 
         # tm.y_test[tm.t:] = tm.sub_predict(tm.X[tm.t:])
         df_ret = df.fillna(method='bfill').pct_change().fillna(0).copy()
+
+        ### getting the transaction costs
+        if self.transaction_costs is None:
+            print('no transaction costs given')
+            costs_parameters = np.zeros(len(df_ret.columns))
+        else:
+            def get_fees_rate(asset, transaction_costs_dic):
+                try:
+                    fees = transaction_costs_dic[asset]
+                except KeyError as e:
+                    print(e)
+                    return 0.
+                return fees
+
+            costs_parameters = np.fromiter(map(lambda x: get_fees_rate(x, self.transaction_costs), df_ret.columns),
+                                           dtype=np.float64)
         # Process weights
         print('processing weights for '+saving_key)
         weight_mat, last_predicted_weights = weights.process_weights(w=tm.y_test, df=df_ret, s=s, n=n, low_bound=low_bound, up_bound=up_bound,normalize=normalize)
         # Compute portfolio perf
-        portfolio = np.cumprod(np.prod(df_ret * weight_mat.values + 1, axis=1))
+        transaction_costs_matrix = abs(weight_mat - weight_mat.shift(1)) *costs_parameters
+        adjusted_returns = df_ret * weight_mat.values - transaction_costs_matrix
+        portfolio = np.cumprod(np.prod(adjusted_returns + 1, axis=1))
         if self.save_model :
             tm.save_model_and_last_state(state=state,model_path = self.local_root_directory+saving_key)
         if self.dropbox_backup:
