@@ -128,7 +128,7 @@ def generate_fourier_wvt_decomposition(lookback_windows,contravariants,n_harms,f
 
     return parameters
 
-def generate_fourier_wvt_ortho_decomposition(lookback_windows,n_harms,fft_filter_types,wavelet_filters,dec_levels,coef_factors):
+def generate_fourier_wvt_ortho_decomposition(lookback_windows,n_harms,fft_filter_types,wavelet_filters,dec_levels,coef_factors, independent_selection_states):
     parameters = []
     for lookback_window in lookback_windows:
         for fft_filter_type in fft_filter_types:
@@ -136,15 +136,17 @@ def generate_fourier_wvt_ortho_decomposition(lookback_windows,n_harms,fft_filter
                 for dec_level in dec_levels:
                     for filter_name in wavelet_filters:
                         for coeff_index_factor in coef_factors:
-                            if n_harm < lookback_window:
-                                parameters.append({
-                                    'lookback_window':lookback_window,
-                                    'fft_filter_type':fft_filter_type,
-                                    'n_harm':n_harm,
-                                    'filter_name':filter_name,
-                                    'dec_level':dec_level,
-                                    'coeff_index_factor':coeff_index_factor
-                                })
+                            for independent_selection in independent_selection_states:
+                                if n_harm < lookback_window:
+                                    parameters.append({
+                                        'lookback_window':lookback_window,
+                                        'fft_filter_type':fft_filter_type,
+                                        'n_harm':n_harm,
+                                        'filter_name':filter_name,
+                                        'dec_level':dec_level,
+                                        'coeff_index_factor':coeff_index_factor,
+                                        'independent_selection':independent_selection
+                                    })
     return parameters
 
 def generate_kalman(lookback_windows,contravariants,n_harms,fft_filter_types,thresholds,obs_covs):
@@ -334,6 +336,31 @@ def generate_slope_lo_induced(lookback_windows, pente_windows, contravariants, l
                                     })
     return parameters
 
+def generate_slope_lo_to_ls(lookback_windows, pente_windows, pente_windows_bis, contravariants, lags, slope_columns, up_trend_thresholds, up_trend_thresholds_bis, display):
+    parameters = []
+    for slope_column in slope_columns:
+        for lookback_window in lookback_windows:
+            for pente_window in pente_windows:
+                for pente_window_bis in pente_windows_bis:
+                    for contravariant in contravariants:
+                        for lag in lags:
+                            for up_trend_threshold_bis in up_trend_thresholds_bis:
+                                for up_trend_threshold in up_trend_thresholds:
+                                    if pente_window < int(lookback_window/lag):
+                                        if lag < int(lookback_window/2):
+                                            parameters.append({
+                                                'lookback_window':lookback_window,
+                                                'pente_window':pente_window,
+                                                'pente_window_bis':pente_window_bis,
+                                                'contravariant':contravariant,
+                                                'up_trend_threshold_bis': up_trend_threshold_bis,
+                                                'up_trend_threshold': up_trend_threshold,
+                                                'lag':lag,
+                                                'slope_column': slope_column,
+                                                'display' : display
+                                            })
+    return parameters
+
 def generate_slope_ls_induced(lookback_windows, pente_windows, contravariants, lags, slope_columns, low_trend_thresholds, up_trend_thresholds, display):
     parameters = []
     for slope_column in slope_columns:
