@@ -12,6 +12,7 @@ import pywt
 import random
 from scipy.ndimage.interpolation import shift
 from napoleontoolbox.wavelets_fft import wavelet_features
+from napoleontoolbox.utility import metrics
 
 continuum_signals = ['alpha_2', 'alpha_3', 'alpha5',  'alpha_6', 'alpha_6_rank', 'alpha_8', 'alpha_12', 'alpha_13', 'alpha_14',  'alpha_15',  'alpha_16', 'slope_induced' ]
 
@@ -860,6 +861,86 @@ def slope_lo_induced(data = None, contravariant = -1., up_trend_threshold=0.8, p
     else:
         return np.nan
 
+def slope_lo_to_ls(data = None, contravariant = -1., up_trend_threshold=0.8, up_trend_threshold_bis=0.8, pente_window = 7, pente_window_bis = 7, lag = 2, slope_column = 'close',**kwargs):
+    ## aggregating the volumes by lag
+    data.index = range(0,len(data))
+    all_indices = list(reversed(range(len(data) - 1, 0, -lag)))
+    data['aggregated_indice'] = np.nan
+    data['aggregated_indice'].loc[all_indices] = all_indices
+    data['aggregated_indice']= data['aggregated_indice'].fillna(method='bfill')
+    data['aggregated_indice'] = data['aggregated_indice'].astype(int)
+    volumefrom = data.groupby(['aggregated_indice'])['volumefrom'].agg('sum')
+    high = data.groupby(['aggregated_indice'])['high'].agg('max')
+    low = data.groupby(['aggregated_indice'])['low'].agg('min')
+
+    data['aggregated_volume'] =  data['aggregated_indice'].map(volumefrom)
+    data['aggregated_high'] =  data['aggregated_indice'].map(high)
+    data['aggregated_low'] =  data['aggregated_indice'].map(low)
+
+    data = data[::-lag]
+    data = data.iloc[::-1]
+
+    data['hl'] = (data['aggregated_high'] - data['aggregated_low'])/data['aggregated_low']
+    data['volu_hi_low'] = data['aggregated_volume']*data['hl']
+
+    data['close_ret']=data['close'].pct_change().fillna(0.)
+    data['high_ret']=data['aggregated_high'].pct_change().fillna(0.)
+
+    data['opp_close_ret']=-data['close'].pct_change().fillna(0.)
+    data['opp_high_ret']=-data['aggregated_high'].pct_change().fillna(0.)
+
+    data['rec_close']=metrics.from_ret_to_price(data['close_ret'])
+    data['rec_opp_close'] =metrics.from_ret_to_price(data['opp_close_ret'])
+
+
+    data['close_pct_rank'] = data['close'].rank(pct=True)
+    data['high_pct_rank'] = data['high'].rank(pct=True)
+
+    data['rec_close_pct_rank'] = data['rec_close'].rank(pct=True)
+    data['rec_opp_close_pct_rank'] = data['rec_opp_close'].rank(pct=True)
+
+    data['high_pct_rank'] = data['high'].rank(pct=True)
+
+    trend_rank = np.nan
+    trend_opp_rank = np.nan
+    if slope_column == 'close':
+        trend_rank = data['close_pct_rank'].iloc[-1]
+        trend_opp_rank = data['rec_opp_close_pct_rank'].iloc[-1]
+    elif slope_column == 'high':
+        raise Exception('not implemented yet')
+
+    slope_close = np.nan
+    slope_opp_close = np.nan
+    slope_rec_close = np.nan
+    if slope_column == 'close':
+        y = data[slope_column][-pente_window:].values
+        slope_close = stats.linregress(np.arange(pente_window), y).slope
+
+        y_rec = data['rec_close'][-pente_window:].values
+        slope_rec_close = stats.linregress(np.arange(pente_window), y_rec).slope
+
+        y_opp = data['rec_opp_close'][-pente_window_bis:].values
+        slope_opp_close = stats.linregress(np.arange(pente_window_bis), y_opp).slope
+
+    elif slope_column == 'high':
+        raise Exception('not implemented yet')
+
+
+    slope_close_condition = (slope_close>0 )
+    slope_rec_close_condition = (slope_rec_close > 0)
+    slope_opp_close_condition = (slope_opp_close > 0)
+    min_slope_test = 0.1
+    if slope_close>min_slope_test and not slope_rec_close_condition == slope_close_condition:
+        print('trouble investigate')
+        assert slope_rec_close_condition == slope_close_condition
+
+    if trend_rank > up_trend_threshold and slope_close>0 :
+        return contravariant
+    elif trend_opp_rank > up_trend_threshold_bis and slope_opp_close>0 :
+        return -contravariant
+    else:
+        return np.nan
+
 def slope_ls(data = None, contravariant = -1., up_trend_threshold=0.8, low_trend_threshold=0.8, pente_window = 7, lag = 2, slope_column = 'high',**kwargs):
     ## aggregating the volumes by lag
     data.index = range(0,len(data))
@@ -1144,9 +1225,10 @@ def signe(a):
     else:
         return -1
 
-def select_orthogonal_coef(wtmra, dec_level, coeff_index_factor):
+def select_orthogonal_coef(wtmra, coeff_index_factor, independent_selection):
     components = wtmra.copy()
-    ortho_coeff = np.zeros(components.shape)
+    independent_ortho_coeff = np.zeros(components.shape)
+    independent_ortho_index = np.zeros(components.shape)
     for j in range(len(components)):
         current_array = np.zeros(components[j].shape)
 
@@ -1162,8 +1244,16 @@ def select_orthogonal_coef(wtmra, dec_level, coeff_index_factor):
                 if -1 * (len(wtmra[0]) + 1) <= a <= len(wtmra[0]):
                     current_array[a - signe(a)] = 1
 
-        ortho_coeff[j] = current_array * components[j]
-    return sum(ortho_coeff)
+        independent_ortho_coeff[j] = current_array * components[j]
+        independent_ortho_index[j] = current_array
+
+        if independent_selection:
+            return sum(independent_ortho_coeff)
+        else:
+            non_independent_indexes = sum(independent_ortho_index)
+            non_independent_indexes[non_independent_indexes != 0] = 1
+            non_independent_ortho_coeff = sum(wtmra)*non_independent_indexes
+            return non_independent_ortho_coeff
 
 def is_discrete(wavelet_to_check, discrete):
     prefix_size = len(wavelet_to_check)
@@ -1221,13 +1311,16 @@ def fourier_wvt_decompo_ls(data=None, filter_name='haar', contravariant=1., dec_
 
 
 def fourier_wvt_ortho_decompo_lo(data=None, filter_name='haar', dec_level=2, n_harm=5,
-                                 fft_filter_type='low', coeff_index_factor=2, **kwargs):
+                                 fft_filter_type='low', coeff_index_factor=2, independent_selection=True, **kwargs):
     n_predict = 1
     try:
-        x = data['return'].values
+        cur_df = data.copy()
+        cur_df['return'] = cur_df['close'].pct_change()
+        cur_df = cur_df.fillna(0)
+        x = cur_df['return'].values
         wt = wavelet_features.modwt(x, filter_name, dec_level)
         decomp_components = wavelet_features.modwtmra(wt, filter_name)
-        approx = select_orthogonal_coef(decomp_components, dec_level, coeff_index_factor)
+        approx = select_orthogonal_coef(decomp_components, coeff_index_factor, independent_selection)
         lb_signal = fourier_extrapolation(approx, n_predict, n_harm, fft_filter_type)
         next_return = lb_signal[0]
         test = np.sign(next_return)
@@ -1238,13 +1331,17 @@ def fourier_wvt_ortho_decompo_lo(data=None, filter_name='haar', dec_level=2, n_h
     except:
         return np.nan
 
-def fourier_wvt_ortho_decompo_ls(data=None, filter_name='haar', dec_level=2, n_harm=5, fft_filter_type='low', coeff_index_factor=2, **kwargs):
+def fourier_wvt_ortho_decompo_ls(data=None, filter_name='haar', dec_level=2, n_harm=5,
+                                 fft_filter_type='low', coeff_index_factor=2, independent_selection=True, **kwargs):
     n_predict = 1
     try:
-        x = data['return'].values
+        cur_df = data.copy()
+        cur_df['return'] = cur_df['close'].pct_change()
+        cur_df = cur_df.fillna(0)
+        x = cur_df['return'].values
         wt = wavelet_features.modwt(x, filter_name, dec_level)
         decomp_components = wavelet_features.modwtmra(wt, filter_name)
-        approx = select_orthogonal_coef(decomp_components, dec_level, coeff_index_factor)
+        approx = select_orthogonal_coef(decomp_components, coeff_index_factor, independent_selection)
         lb_signal = fourier_extrapolation(approx, n_predict, n_harm, fft_filter_type)
         next_return = lb_signal[0]
         return np.sign(next_return)
